"
 COPYRIGHT (c) 2011 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libbasic' }"

Timestamp subclass:#UtcTimestamp
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Magnitude-Time'
!

!UtcTimestamp class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2011 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.

"
!

documentation
"
    This is a Timestamp, that represents itself as UTC.
    Internally it stores exactly the same value as Timestamp, 
    so the following expression is always true:

        someTimestamp asUtcTimestamp = someTimestamp asLocalTimestamp

    But when printing, accessing hour, minute, ... and converting to Date and Time,
    it returns the values represented in the UTC zone instead of the local timezone.

    [author:]
        Stefan Vogel (stefan@zwerg.nil.exept.de)

    [instance variables:]

    [class variables:]

    [see also:]
        Time Date

"
! !

!UtcTimestamp methodsFor:'accessing'!

timeInfo
    ^ self utcTimeInfo
!

timeZoneName
    ^ 'UTC'
!

utcOffset
    ^ 0
! !

!UtcTimestamp methodsFor:'converting'!

asDate
    "return a Date object from the receiver.
     The returned date will only represent the day - not the timeOfDay.
     Notice: if you convert a local timestamp, you will get the local date;
     otherwise if you convert an utcTimestamp, you'll get the utc date."

    (osTime between:MinOSTime and:MaxOSTime) ifFalse:[
        |dayDelta day0|

        dayDelta := osTime // (24 * 3600 * 1000).
        day0 := Epoch asDate.
        ^ day0 addDays:dayDelta.
    ].
    ^ self timeInfo asDate

    "
     Timestamp now
     Timestamp now asDate
     (Timestamp now addTime:3600) asDate
     (Timestamp now addTime:3600) asTime
     Timestamp fromSeconds:(Timestamp now asSeconds + 3600)
     (Timestamp fromSeconds:(Timestamp now asSeconds + 3600)) asDate
    "
!

asLocalTimestamp
    "represent myself as a timestamp in the local timezone"

    ^ Timestamp fromOSTime:osTime
!

asTime
    "return a Time object from the receiver.
     The returned time will only represent the timeOfDay - not the day,
     and does not include the milliseconds."

    |secondDelta|

    (osTime between:MinOSTime and:MaxOSTime) ifFalse:[
        secondDelta := osTime // 1000.
        "/ we do not know about DST in the far future and in the long gone past.
        secondDelta := secondDelta rem:(24*3600) .
        ^ Epoch asTime addSeconds:secondDelta.
    ].
    ^ self timeInfo asTime
!

asUtcTimestamp
    "I am an UtcTimestamp"

    ^ self
! !

!UtcTimestamp methodsFor:'initialization'!

setOSTimeFromYear:y month:m day:d hour:h minute:min second:s millisecond:millis 
    "private: ask the operating system to compute the internal osTime (based on the epoch),
     given y,m,d and h,m,s in utc time"

   self setOSTimeFromUTCYear:y month:m day:d hour:h minute:min second:s millisecond:millis

    "
     UtcTimestamp now            
     Timestamp now              
     Timestamp now asTZTimestamp             
     Timestamp now asUtcTimestamp  
    "

    "Modified (comment): / 13-10-2011 / 22:05:41 / cg"
! !

!UtcTimestamp methodsFor:'testing'!

isLocalTimestamp
    "return true, if I am a local timestamp"

    ^ false
!

isUtcTimestamp
    "return true, if I am a utc timestamp"

    ^ true
! !

!UtcTimestamp methodsFor:'visiting'!

acceptVisitor:aVisitor with:aParameter
    "dispatch for visitor pattern; send #visitUtcTimestamp:with: to aVisitor."

    ^ aVisitor visitUtcTimestamp:self with:aParameter
! !

!UtcTimestamp class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic/UtcTimestamp.st,v 1.12 2014-11-11 12:14:23 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/libbasic/UtcTimestamp.st,v 1.12 2014-11-11 12:14:23 stefan Exp $'
! !

